#!/usr/bin/env python
# -*- coding: utf-8 -*-


import numpy as np
import pylab
import re
import os

import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
# http://matplotlib.org/examples/axes_grid/demo_colorbar_with_inset_locator.html

from optparse import OptionParser

STATE_FILENAME = "orbit.log"
SLICE = 1
T_MIN = 0
T_MAX = 300
PHASE = 0
C_MAP = plt.cm.seismic
params = {"backend": "pdf",
          "font.family": "Arial",
          "axes.titlesize": 20,
          "axes.labelsize": 20,
          "text.fontsize": 20,
          "legend.fontsize":20,
          "xtick.labelsize": 20,
          "ytick.labelsize": 20,
          "text.usetex": False,
          "savefig.facecolor": "1.0"}
pylab.rcParams.update(params)

def read_parameter(f):
    r = {}
    r["c_state_size"] = re.compile(r"^# c_state_size")
    r["out_state_size"] = re.compile(r"^# out_state_size")
    r_comment = re.compile(r'^#')
    params = {}
    for line in f:
        for k,v in r.iteritems():
            if (v.match(line)):
                x = line.split('=')[1]
                if k == 'target':
                    m = int(v.match(line).group(1))
                    if (k in params):
                        params[k][m] = x
                    else:
                        params[k] = {m:x}
                else:
                    params[k] = x

        if (r_comment.match(line) == None):
            break
    f.seek(0)
    return params

c_state_size = 100
out_state_size = 10

class plot_rnn(object):
    def __init__(self,
                 filename):
        self.state_filename = filename
        self.figure_name = self.state_filename.replace(".log", ".pdf")

    def add_info(self, ax, title, xlim, ylim, xlabel, ylabel):
        if title != None:
            ax.set_title(title)

        if xlim != None:
            ax.set_xlim(xlim)
            #ax.set_xticks((xlim[0], (xlim[0] + xlim[1]) / 2.0, xlim[1]))
            ax.set_xticks([0, 120, 200, 300])
        else:
            ax.set_xticks([])

        if xlabel != None:
            ax.set_xlabel(xlabel)

        if ylim != None:
            ax.set_ylim(ylim)
            ax.set_yticks((ylim[0], (ylim[0] + ylim[1]) / 2.0, ylim[1]))
        if ylabel != None:
            ax.set_ylabel(ylabel)

        ax.grid(True) 
        
    def set_no_yticks(self, ax):
        ax.set_yticks([])

    def configure(self, fig_matrix, width, height):
        fig = plt.figure(figsize = (width * fig_matrix[1], height * fig_matrix[0]))
        gs = gridspec.GridSpec(fig_matrix[0], fig_matrix[1])
        axes = [fig.add_subplot(gs[i, j]) for i in xrange(fig_matrix[0]) for j in xrange(fig_matrix[1])]
        #fig.subplots_adjust(hspace=0.1)
        #axes[0] = fig.add_subplot
        return fig, gs, axes

    def plot_colormap(self, ax, state, range):
        im = ax.imshow(state.T, vmin = range[0], vmax = range[1], aspect = "auto", interpolation = "nearest", cmap = C_MAP)
        if state.shape[0] is not 1:
            ax.set_xlim(0, state.shape[0])
            ax.set_ylim(-0.5, state.shape[1] - 0.5)
            ax.set_yticks((0, state.shape[1] -1))
        return im

    def state(self, slice_num, tmin, tmax, use_colormap, width, height):
        fig_matrix = [4, 1]
        fig, gs, axes = self.configure(fig_matrix, width, height)
        delay_length = 5

        data = np.loadtxt(self.state_filename)
        teach_state = data[:, :out_state_size]
        c_state = np.tanh(data[:, 3 * out_state_size: 3 * out_state_size + c_state_size])
        #print c_state_size
        #c_state = np.tanh(data[:, -4:])
        in_state = data[:, :out_state_size]
        out_state = data[:, out_state_size: 2 * out_state_size]
        var_state = data[:, 2 * out_state_size: 3 * out_state_size]
        pb_state = np.tanh(data[:, 3 * out_state_size + c_state_size:])
        likelihood = 0.5 * np.log(2 * np.pi * (var_state[:-delay_length, :]+0.0)) + 0.5 * (in_state[delay_length:, :] - out_state[:-delay_length, :]) * (in_state[delay_length:, :] - out_state[:-delay_length, :]) / (var_state[:-delay_length, :]+0.0)


        axes[0].plot(out_state[:, 3], color="red", linestyle="solid", linewidth="3")
        axes[0].plot(out_state[:, 7], color="blue", linestyle="solid", linewidth="3")
        self.add_info(axes[0], None, None, (-1,1), None, None)
        axes[1].plot(in_state[:, 8], color="gold", linestyle="dashed", linewidth="3")
        axes[1].plot(in_state[:, 9], color="gold", linestyle="solid", linewidth="3")
        self.add_info(axes[1], None, None, (-1,1), None, None)
        axes[2].plot(likelihood[:, 8], color="gold", linestyle="dashed", linewidth="3")
        axes[2].plot(likelihood[:, 9], color="gold", linestyle="solid", linewidth="3")
        self.add_info(axes[2], None, None, (0, 1000), None, None)

        
        if use_colormap:
            range = [-1, 1]
            im = self.plot_colormap(axes[3], pb_state[:, :], range)
            self.add_info(axes[3], None, (-1.1), None, None, None)
            if pb_state.shape[1] is 1:
                self.set_no_yticks(axes[3])
            axins = inset_axes(axes[3],
                               width = "5%", # width = 5% of parent_bbox width
                               height = "100%", # height : 50%
                               loc = 3,
                               bbox_to_anchor = (1.05, 0, 1, 1),
                               bbox_transform = axes[3].transAxes,
                               borderpad = 0,
                               )
            plt.colorbar(im, cax = axins, ticks = (-1, 0, 1))
            gs.tight_layout(fig, rect = [0, 0, 0.9, 1]) # left, bottom, right, top

        else:
            axes[3].plot(pb_state[:, :])
            self.add_info(axes[3], None, None, (-1, 1), "Time step", "Context")
            gs.tight_layout(fig, rect = [0, 0, 1, 1]) # left, bottom, right, top

        for ax in axes:
            ax.set_xlim(tmin, tmax)

        fig.savefig(self.figure_name, format="pdf",dpi=600)
        fig.show()

    def plot_phase(self, phase_plot_num, ax, state, label):
        for i in xrange(state.shape[1] / 2):
            if i == phase_plot_num:
                break            
            ax.plot(state[:, 2 * i], state[:, 2 * i + 1])
        self.add_info(ax, label, (-1, 1), (-1, 1), "i-th dim.", "(i+1)-th dim.")

    def phase(self, phase_plot_num, tmin, tmax, width, height):
        fig_matrix = [1, 3]
        fig, gs, axes = self.configure(fig_matrix, width, height)

        data = np.loadtxt(self.state_filename)
        if (tmin is not None) and (tmax is not None):
            data = data[tmin:tmax, :]
        elif tmin is not None:
            data = data[tmin:, :]
        elif tmax is not None:
            data = data[:tmax, :]

        teach_state = data[:, 1:1 + 3 * out_state_size:3]
        c_state = np.tanh(data[:, 1 + 3 * out_state_size:1 + 3 * out_state_size + c_state_size])
        out_state = data[:, 2:1 + 3 * out_state_size:3]

        for ax in axes:
            ax.set_aspect("equal")
        if teach_state.shape[1] is not 1:
            self.plot_phase(phase_plot_num, axes[0], teach_state, "Target")
        if out_state.shape[1] is not 1:
            self.plot_phase(phase_plot_num, axes[1], out_state, "Output")
        if c_state.shape[1] is not 1:
            self.plot_phase(phase_plot_num, axes[2], c_state, "Context")
        
        gs.tight_layout(fig, rect = [0, 0, 1, 1]) # left, bottom, right, top
        fig.savefig(self.figure_name.replace(".pdf", "_phase.pdf"))
        fig.show()

def main():
    parser = OptionParser()
    parser.add_option("-f", "--file", dest="filename",
                      help="filename for plot", metavar="FILENAME", default=STATE_FILENAME)
    parser.add_option("-s", "--slice", dest="slice_num",
                      help="slice value for easy understanding", metavar="SLICE", default=SLICE)
    parser.add_option("-m", "--minimum", dest="tmin",
                      help="minimum value of time axis", metavar="MIN", default=T_MIN)
    parser.add_option("-M", "--maximum", dest="tmax",
                      help="maximum value of time axis", metavar="MAX", default=T_MAX)
    parser.add_option("-p", "--phase", dest = "phase_plot_num", metavar = "PHASE",
                      help="number of phase plot", default = PHASE)
    parser.add_option("-c", "--colormap", dest = "use_colormap", action = "store_false",
                      help="use colormap for context or not", default = True)

    (options, args) = parser.parse_args()

    filename = options.filename
    slice_num = int(options.slice_num)
    if options.tmin is None:
        tmin = options.tmin
    else:
        tmin = int(options.tmin)
    if options.tmax is None:
        tmax = options.tmax
    else:
        tmax = int(options.tmax)
    phase_plot_num = int(options.phase_plot_num)
    use_colormap = options.use_colormap

    plot = plot_rnn(filename)
    
    plot.state(slice_num, tmin, tmax, use_colormap, 5, 2)
    if 0 < phase_plot_num:
        plot.phase(phase_plot_num, tmin, tmax, 3, 3.1)
    
    raw_input()

if __name__ == "__main__":
    main()
